#include "mqtt.h"
#include "mgos_gpio.h"
#include "buttons.h"
#include "relays.h"
#include "helper.h"

struct button_t s_buttons[MAX_BUTTONS];

void buttons_cb(int gpio, void *arg) {
  int idx = buttons_find_by_gpio(gpio);

  if (ERR_GPIO_INVALID == idx) {
    LOG(LL_ERROR, ("No button on gpio=%d", gpio));
    return;
  }
  LOG(LL_INFO, ("Button on gpio=%d idx=%d", gpio, idx));

  buttons_touch_by_idx(idx);
  return;
  (void) arg;
}

int buttons_init(const char *flag) {
  int next_button=0;
  int i;

  memset(s_buttons, -1, sizeof(s_buttons));

  LOG(LL_DEBUG, ("Flag='%s'", flag));
  if (!is_button_flag_valid(flag)) {
    return -1;
  }

  i=0;
  while(i<(int) strlen(flag)) {
    int gpio = -1;

    if (!isdigit((int) flag[i])) {
      i++;
      continue;
    }

    gpio = atoi((char *)flag+i);
//    LOG(LL_DEBUG, ("Token: %d at pos %u", gpio, i));
    if (gpio<MIN_GPIO || gpio>MAX_GPIO) {
      LOG(LL_ERROR, ("GPIO values must be within [%d,%d], gpio=%d is invalid", MIN_GPIO, MAX_GPIO, gpio));
      return -2;
    }
    s_buttons[next_button].gpio=gpio;
    s_buttons[next_button].last_change=-1;
    mgos_gpio_set_mode(gpio, MGOS_GPIO_MODE_INPUT);
    mgos_gpio_set_button_handler(gpio, MGOS_GPIO_PULL_UP, MGOS_GPIO_INT_EDGE_NEG, 100, buttons_cb, NULL);

    next_button++;
    if (next_button>MAX_BUTTONS) {
      LOG(LL_ERROR, ("Too many buttons defined, max is %d", MAX_BUTTONS));
      return -3;
    }

    while (i<(int)strlen(flag) && isdigit((int) flag[i])) i++;
  }

  return 0;
}

uint8_t buttons_get_gpio_by_idx(uint8_t idx) {
  if (idx>=MAX_BUTTONS)
    return ERR_GPIO_INVALID;
  return s_buttons[idx].gpio;
}



uint8_t buttons_find_by_gpio(uint8_t gpio) {
  int i;
  for (i=0; i<MAX_BUTTONS; i++) {
    if (gpio==s_buttons[i].gpio)
      return i;
  }
  return ERR_GPIO_INVALID;
}

bool buttons_touch_by_idx(uint8_t idx) {
  char msg[80];
  int relay_gpio;

  if (idx>=MAX_BUTTONS || s_buttons[idx].gpio == ERR_GPIO_INVALID) {
    LOG(LL_ERROR, ("No button at idx=%d", idx));
    return false;
  }
  s_buttons[idx].last_change=mg_time();

  snprintf(msg, sizeof(msg)-1, "{\"idx\": %d, \"gpio\": %d}", idx, s_buttons[idx].gpio);
  mqtt_publish_stat("button", msg);

  // Lookup corresponding relay at the same index. If it exists, trigger it.
  relay_gpio = relays_get_gpio_by_idx(idx);
  if (ERR_GPIO_INVALID == relay_gpio)
    return true;

  relays_set_by_gpio(relay_gpio, !relays_get_by_gpio(relay_gpio));
  return true;
}

bool buttons_touch_by_gpio(uint8_t gpio) {
  return buttons_touch_by_idx(buttons_find_by_gpio(gpio));
}

float buttons_get_last_change_by_gpio(uint8_t gpio) {
  return buttons_get_last_change_by_idx(buttons_find_by_gpio(gpio));
}

float buttons_get_last_change_by_idx(uint8_t idx) {
  if (idx>=MAX_BUTTONS || s_buttons[idx].gpio == ERR_GPIO_INVALID) {
    LOG(LL_ERROR, ("No button at idx=%d", idx));
    return -1;
  }
  return s_buttons[idx].last_change;
}
