#include "mgos_gpio.h"
#include "mqtt.h"
#include "relays.h"
#include "helper.h"

struct relay_t s_relays[MAX_RELAYS];

int relays_init(const char *flag) {
  int next_relay=0;
  int i;

  memset(s_relays, -1, sizeof(s_relays));

  LOG(LL_DEBUG, ("Flag='%s'", flag));
  if (!is_relay_flag_valid(flag)) {
    return -1;
  }

  i=0;
  while(i<(int) strlen(flag)) {
    int gpio = -1;

    if (!isdigit((int) flag[i])) {
      i++;
      continue;
    }

    gpio = atoi((char *)flag+i);
//    LOG(LL_DEBUG, ("Token: %d at pos %u", gpio, i));
    if (gpio<MIN_GPIO || gpio>MAX_GPIO) {
      LOG(LL_ERROR, ("GPIO values must be within [%d,%d], gpio=%d is invalid", MIN_GPIO, MAX_GPIO, gpio));
      return -2;
    }
    s_relays[next_relay].gpio=gpio;
    s_relays[next_relay].last_change=-1;
    s_relays[next_relay].state=0;
    mgos_gpio_set_mode(gpio, MGOS_GPIO_MODE_OUTPUT);
    mgos_gpio_write(gpio, !s_relays[next_relay].state);
    next_relay++;
    if (next_relay>MAX_RELAYS) {
      LOG(LL_ERROR, ("Too many relays defined, max is %d", MAX_RELAYS));
      return -3;
    }

    while (i<(int)strlen(flag) && isdigit((int) flag[i])) i++;
  }

  return 0;
}

uint8_t relays_get_gpio_by_idx(uint8_t idx) {
  if (idx>=MAX_RELAYS)
    return ERR_GPIO_INVALID;
  return s_relays[idx].gpio;
}

uint8_t relays_find_by_gpio(uint8_t gpio) {
  int i;
  for (i=0; i<MAX_RELAYS; i++) {
    if (gpio==s_relays[i].gpio)
      return i;
  }
  return ERR_GPIO_INVALID;
}

bool relays_set_by_gpio(uint8_t gpio, bool state) {
  return relays_set_by_idx(relays_find_by_gpio(gpio), state);
}

bool relays_set_by_idx(uint8_t idx, bool state) {
  char msg[60];

  if (idx>=MAX_RELAYS || s_relays[idx].gpio == ERR_GPIO_INVALID) {
    LOG(LL_ERROR, ("No relay at idx=%d", idx));
    return false;
  }
  s_relays[idx].last_change=mg_time();
  if (state != s_relays[idx].state) {

    LOG(LL_INFO, ("Setting relay idx=%d gpio=%d to %s", idx, s_relays[idx].gpio, state==true?"ON":"OFF"));
    s_relays[idx].state=state;
    mgos_gpio_write(s_relays[idx].gpio, !s_relays[idx].state);
  }

  snprintf(msg, sizeof(msg)-1, "{\"idx\": %d, \"gpio\": %d, \"state\": %d}", idx, s_relays[idx].gpio, state);
  mqtt_publish_stat("relay", msg);
  return true;
}

bool relays_get_by_gpio(uint8_t gpio) {
  return relays_get_by_idx(relays_find_by_gpio(gpio));
}

bool relays_get_by_idx(uint8_t idx) {
  if (idx>=MAX_RELAYS || s_relays[idx].gpio == ERR_GPIO_INVALID) {
    LOG(LL_ERROR, ("No relay at idx=%d", idx));
    return false;
  }
  return s_relays[idx].state;
}

float relays_get_last_change_by_gpio(uint8_t gpio) {
  return relays_get_last_change_by_idx(relays_find_by_gpio(gpio));
}

float relays_get_last_change_by_idx(uint8_t idx) {
  if (idx>=MAX_RELAYS || s_relays[idx].gpio == ERR_GPIO_INVALID) {
    LOG(LL_ERROR, ("No relay at idx=%d", idx));
    return -1;
  }
  return s_relays[idx].last_change;
}
